import os
import logging
from dotenv import load_dotenv


class EnvConfig:
    @staticmethod
    def load_dotenv():
        """
        Carga las variables de entorno desde un archivo .env en la misma ubicación que el módulo.
        """
        # Obtén la ruta absoluta del directorio del módulo
        module_dir = os.path.dirname(os.path.abspath(__file__))

        # Calcula la ruta absoluta al archivo .env en la misma ubicación que el módulo
        env_file_path = os.path.join(module_dir, '.env')
        # print(env_file_path)
        # Carga las variables de entorno desde el archivo .env
        load_dotenv(env_file_path)

        # Establece el nivel de registro (log level) en base a CPOC_LOG_LEVEL
        log_level = os.getenv('CPOC_LOG_LEVEL', 'DEBUG')
        logging.basicConfig(level=log_level)

        # Muestra todas las variables de entorno en modo de depuración
        logger = logging.getLogger(__name__)
        for key, value in os.environ.items():
            if key.startswith('CPOC_'):
                logger.debug(f'{key}={value}')

    @staticmethod
    def get_variable(variable_name, default=None):
        """
        Recupera el valor de una variable de entorno.

        :param variable_name: Nombre de la variable de entorno.
        :param default: Valor por defecto a retornar si la variable no está definida.
        :return: El valor de la variable de entorno o el valor por defecto si no está definida.
        """
        if variable_name in os.environ:
            return os.getenv(variable_name, default)
        else:
            logging.warning(
                f"La variable de entorno '{variable_name}' no existe. Se retornará el valor default '{default}'")
            return default



from ...utils import Job
from . import jtask

description = "Ejemplo de job test para plantilla de desrrollo."
configMap = {
    "type": "object",
    "properties": {
        "nombreArchivo": {
            "type": "string",
            "description": "El nombre del archivo."
        },
        "ruta": {
            "type": "string",
            "description": "La ruta del archivo."
        }
    },
    "required": ["nombreArchivo", "ruta"]
}


class MJob(Job.Job):
    def __init__(self):
        super().__init__(description,
                         configMap)

    def task(self):
        jtask.run(self.params)


def get():
    return MJob()



import sqlite3
import json
from . import CharacterAudio
import os
import logging
from concurrent.futures import ThreadPoolExecutor


class CharacterAudioRender:

    def __init__(self, character_id, db_file):
        self.db_file = db_file
        self.character_id = character_id
        self._load()
        self.logger = logging.getLogger(__name__)

    def _load(self):
        conn = sqlite3.connect(self.db_file)
        conn.row_factory = sqlite3.Row
        cursor = conn.cursor()
        query = '''
        SELECT Character.character_id, name, tts
        FROM Character
        INNER JOIN CharacterDescriptor ON Character.character_id = CharacterDescriptor.character_id
        WHERE Character.character_id = ?;
        '''
        cursor.execute(query, (self.character_id,))
        character_data = cursor.fetchone()
        conn.close()

        if character_data:
            self.name = character_data['name']
            self.tts = json.loads(character_data['tts'])
            self.audio = CharacterAudio.CharacterAudio(config=self.tts)
            return True
        else:
            return False

    def render_dialogue(self, dialogue_id, text, output_path):
        try:
            self.audio.say(text, out=output_path)

            conn = sqlite3.connect(self.db_file)
            cursor = conn.cursor()
            update_query = '''
            UPDATE Dialogue
            SET audioPath = ?
            WHERE dialogue_id = ?;
            '''
            cursor.execute(update_query, (output_path, dialogue_id))
            conn.commit()
            conn.close()
            self.logger.debug(
                f"Rendered dialogue {dialogue_id} to {output_path}")
        except Exception as e:
            self.logger.error(
                f"Error rendering dialogue {dialogue_id}: {str(e)}")

    def renderAllDialogs(self, outputDirectory, maxjobs=1, outprefix=""):
        conn = sqlite3.connect(self.db_file)
        conn.row_factory = sqlite3.Row
        cursor = conn.cursor()
        query = '''
        SELECT dialogue_id, text
        FROM Dialogue
        WHERE character_id = ?;
        '''
        cursor.execute(query, (self.character_id,))
        dialogues = cursor.fetchall()
        conn.close()

        self.logger.info(f"Started rendering {len(dialogues)} dialogues.")

        with ThreadPoolExecutor(maxjobs) as executor:
            futures = []
            for dialogue in dialogues:
                dialogue_id = dialogue['dialogue_id']
                text = dialogue['text']
                output_path = os.path.join(
                    outputDirectory, f"{outprefix}{self.character_id}_{dialogue_id}.wav")
                futures.append(executor.submit(
                    self.render_dialogue, dialogue_id, text, output_path))

            for future in futures:
                future.result()

        self.logger.info("Finished rendering all dialogues.")


"""
if __name__ == "__main__":
    # Example usage:
    character_id = 1
    db_file = "/Users/alvaroperis/Dropbox/cinemawritter/fontana/miguion.cst"
    output_dir = "/Users/alvaroperis/Dropbox/cinemawritter/fontana/salidas"

    # Configura el nivel de log a DEBUG
    logging.basicConfig(level=logging.DEBUG,
                        format='%(asctime)s - %(levelname)s - %(message)s')

    renderer = CharacterAudioRender(character_id, db_file)
    renderer.renderAllDialogs(output_dir)
"""



import sqlite3
palabras_por_minuto = 130


class Descriptor:
    def __init__(self, db_file):
        self.db_file = db_file

    def get_script_info(self):
        conn = sqlite3.connect(self.db_file)
        cursor = conn.cursor()

        # Obtener información general del guión (script)
        cursor.execute("SELECT COUNT(DISTINCT name) FROM Character")
        total_caracteres = cursor.fetchone()[0]

        cursor.execute("SELECT COUNT(*) FROM Scene")
        total_scenes = cursor.fetchone()[0]

        cursor.execute("SELECT COUNT(*) FROM Dialogue")
        total_lineas_dialogo = cursor.fetchone()[0]

        cursor.execute(
            "SELECT SUM(LENGTH(text) - LENGTH(REPLACE(text, ' ', '')) + 1) FROM Dialogue")
        total_de_palabras = cursor.fetchone()[0]

        conn.close()

        return {
            "stats": {
                "totalCaracteres": total_caracteres,
                "totalScenes": total_scenes,
                "totalLineasDialogo": total_lineas_dialogo,
                "totalDePalabras": total_de_palabras,
                "estimacionTiempo": round(total_de_palabras / palabras_por_minuto, 1)
            },
            "scenes": self.get_scene_info(),
            "caracteres": self.get_character_info(),
        }

    def get_scene_info(self):
        conn = sqlite3.connect(self.db_file)
        cursor = conn.cursor()

        # Obtener información de las escenas (scenes)
        cursor.execute(
            "SELECT scene_id, location FROM Scene ORDER BY scene_id ASC")
        scenes = cursor.fetchall()

        scene_info = []

        for scene in scenes:
            scene_id = scene[0]
            location = scene[1]

            # Calcular el número de palabras en la escena actual
            cursor.execute(
                "SELECT SUM(LENGTH(text) - LENGTH(REPLACE(text, ' ', '')) + 1) FROM Dialogue WHERE scene_id = ?", (scene_id,))
            palabras_escena = cursor.fetchone()[0]

            # Calcular la duración estimada de la escena actual
            if palabras_escena is None:
                palabras_escena = 0
            minutos_estimados = round(palabras_escena / palabras_por_minuto, 2)

            scene_info.append({
                "pos": scene_id,
                "name": location,
                "words": palabras_escena,
                "time": minutos_estimados
            })

        conn.close()

        return scene_info

    def get_character_info(self):
        conn = sqlite3.connect(self.db_file)
        cursor = conn.cursor()

        # Obtener información de los caracteres (characters)
        cursor.execute("SELECT name FROM Character")
        characters = cursor.fetchall()

        conn.close()

        return [character[0] for character in characters]

    def get_characters(self):
        conn = sqlite3.connect(self.db_file)
        # Configurar el cursor para retornar resultados como diccionarios
        conn.row_factory = sqlite3.Row
        cursor = conn.cursor()
        cursor.execute("SELECT * FROM Character")
        characters = cursor.fetchall()
        conn.close()
        return characters

    def get_scenes(self):
        conn = sqlite3.connect(self.db_file)
        # Configurar el cursor para retornar resultados como diccionarios
        conn.row_factory = sqlite3.Row
        cursor = conn.cursor()
        cursor.execute("SELECT * FROM Scene")
        scenes = cursor.fetchall()
        conn.close()
        return scenes


# Uso de la clase Descriptor
if __name__ == "__main__":
    db_file_path = "ejemplo.cst"
    descriptor = Descriptor(db_file_path)
    script_info = descriptor.get_script_info()
    print(script_info)



import os
import sqlite3
import logging


class DatabaseAlreadyExistsError(Exception):
    pass


def createDB(name):
    if not os.path.exists(f"{name}.cst"):
        raise DatabaseAlreadyExistsError(
            f"El archivo {name}.cst no existe. Es necesario iniciar un proyecto cst previamente.")

    conn = sqlite3.connect(f"{name}.cst")
    # Crear un cursor para ejecutar comandos SQL
    cursor = conn.cursor()

    # Eliminar las tablas si ya existen
    cursor.execute('''DROP TABLE IF EXISTS CharacterDescriptor;''')
    cursor.execute('''DROP TABLE IF EXISTS Timeline;''')
    cursor.execute('''DROP TABLE IF EXISTS Tags;''')

    # Sentencia SQL para crear la tabla Character
    cursor.execute('''
    CREATE TABLE IF NOT EXISTS CharacterDescriptor (
        character_id INTEGER PRIMARY KEY,
        tts TEXT
    );
    ''')

    # Sentencia SQL para crear la tabla de Timeline
    cursor.execute('''
    CREATE TABLE IF NOT EXISTS Timeline (
        sequence_id INTEGER PRIMARY KEY,
        type TEXT,
        content TEXT
    );
    ''')

    # Sentencia SQL para crear la tabla de Timeline
    cursor.execute('''
    CREATE TABLE IF NOT EXISTS Tags (
        tag_id INTEGER PRIMARY KEY,
        name TEXT,
        value TEXT
    );
    ''')

    # Guardar los cambios y cerrar la conexión
    conn.commit()
    conn.close()
    logging.debug("Tablas de timeline creadas")



import os
import sqlite3
import logging
import sys
from ...lib.scene import SceneAudioRender


def run(params):
    logger = logging.getLogger(__name__)

    # 1. Mira si existe el directorio indicado en `params['storepath']`.
    # Si no existe, créalo y notifica con un logger.debug que lo ha creado.
    storepath = params['storepath']
    if not os.path.exists(storepath):
        os.makedirs(storepath)
        logger.debug(f"Created directory: {storepath}")

    # 2. Dentro de este directorio crea una carpeta llamada `scenes`
    scenes_dir = os.path.join(storepath, "secenes")
    if not os.path.exists(scenes_dir):
        os.makedirs(scenes_dir)

    # 3. Se conecta a la sqlite3 especificada en `params['pname']`.
    db_file = f"{params['pname']}.cst"
    if not os.path.exists(db_file):
        logger.error(f"The project {db_file} does not exist.")
        return

    conn = sqlite3.connect(db_file)
    cursor = conn.cursor()

    # 4. Obtiene el `scene_id` de todos los personajes.
    cursor.execute('SELECT scene_id FROM Scene;')
    scene_ids = cursor.fetchall()
    conn.close()

    for scene_id in scene_ids:
        scene_name = f"scene_{str(scene_id[0])}"
        scene_id = scene_id[0]
        output_dir = os.path.join(scenes_dir, str(scene_name))
        if not os.path.exists(output_dir):
            os.makedirs(output_dir)

        renderer = SceneAudioRender.SceneAudioRender(
            scene_id, db_file)
        if renderer.isDialogueRendered():
            renderer.renderScene(output_dir)
        else:
            print(
                f"Error, los dialogos de la scena {str(scene_id)} no se han renderizado. Revise si se han renderizado los dialogos ")
            sys.exit(1)


"""
if __name__ == "__main__":
    # Example usage:
    params = {
        'storepath': "/path/to/store",
        'pname': "/path/to/project.db"
    }
    run(params)
"""



import jsonschema
import logging
import sys


class Job:
    def __init__(self, description="", configMap={}):
        """
        Constructor de la clase Job.

        :param description: Descripción del trabajo.
        :param configMap: Diccionario que representa un JSON Schema.
        """
        self.description = description
        self.configMap = configMap
        self.params = {}

    def __str__(self):
        return f"Job(description='{self.description}', configMap={self.configMap})"

    def _validate_config(self, params):
        """
        Valida el diccionario params con un JSON Schema.
        """
        try:
            jsonschema.validate(instance=params, schema=self.configMap)
            return True
        except jsonschema.exceptions.ValidationError as e:
            logging.error("Error de validación del JSON Schema: %s", e)
            # Obtener la lista de campos faltantes
            campos_faltantes = [err.split("'")[1] for err in e.validator_value]
            logging.error(campos_faltantes)
            return False

    def task(self):
        logging.warning(
            f"Este job no realiza ninguna task. Implemente la función `task()`")

    def getParam(self, param):
        return self.params[param]

    def run(self, params):
        if self.configMap:
            if not self._validate_config(params):
                logging.error("Error al ejecutar el job")
                sys.exit(1)
            else:
                self.params = params
        logging.debug(f"Lanzando job con los parametros {str(params)}")
        self.task()



import sqlite3
import json
from pydub import AudioSegment


class SceneAudioRender:

    def __init__(self, scene_id, db_file):
        self.db_file = db_file
        self.scene_id = scene_id
        self._load()

    def _load(self):
        conn = sqlite3.connect(self.db_file)
        cursor = conn.cursor()
        cursor.execute(
            "SELECT dialogue_id, audioPath FROM Dialogue WHERE scene_id = ? ORDER BY dialogue_id ASC", (self.scene_id,))
        self.dialogue_records = cursor.fetchall()
        conn.close()

    def isDialogueRendered(self):
        """
        Valida que el campo `audioPath` esté informado en todos los registros de la tabla `Dialogue`
        para el `scene_id` especificado.
        """
        conn = sqlite3.connect(self.db_file)
        cursor = conn.cursor()
        cursor.execute(
            "SELECT COUNT(*) FROM Dialogue WHERE scene_id = ? AND audioPath IS NULL", (self.scene_id,))
        count = cursor.fetchone()[0]
        conn.close()

        if count == 0:
            return True  # Todos los registros tienen audioPath informado
        else:
            return False  # Al menos un registro no tiene audioPath informado

    def renderScene(self, outputDirectory, outprefix=""):
        concatenated_audio = AudioSegment.silent(
            duration=0)  # Inicializar un audio vacío

        for dialogue_id, audioPath in self.dialogue_records:
            audio = AudioSegment.from_file(audioPath, format="wav")
            concatenated_audio += audio

        output_path = f"{outputDirectory}/{outprefix}{self.scene_id}.wav"
        concatenated_audio.export(output_path, format="wav")

        # Actualizar la tabla Scene con la ruta de salida
        conn = sqlite3.connect(self.db_file)
        cursor = conn.cursor()
        cursor.execute(
            "UPDATE Scene SET audioPath = ? WHERE scene_id = ?", (output_path, self.scene_id))
        conn.commit()
        conn.close()



from . import TimelineTemplate
from ...lib import Descriptor


def run(params):
    db_file_path = params['pname']
    output_file_path = f"{params['pname']}.yaml"

    descriptor = Descriptor.Descriptor(db_file_path)
    characters = descriptor.get_characters()
    scenes = descriptor.get_scenes()

    # Renderiza la plantilla con los datos
    rendered_yaml = TimelineTemplate.TimelineTemplate(
        params['useDefault']).render(characters=characters, scenes=scenes)

    # Guarda el resultado en un archivo YAML

    with open(output_file_path, "w") as output_file:
        output_file.write(rendered_yaml)

    print(f"Archivo YAML generado y guardado en {output_file_path}")



from . import dbjob
from . import loadjob


def run(params):
    # Creamos la base de datos
    dbjob.createDB(params['pname'])
    # Cargamos el fichero fountain
    loadjob.runloader(params['timeline'], params['pname'])



# cassette/main.py

import click
from cassette.commands import run, list_jobs
from cassette.utils import EnvConfig

EnvConfig.EnvConfig.load_dotenv()


@click.group()
def cli():
    """
    🎬 Cassette 📼 - Aplicación para renderizar escenas desde archivos Fountain con configuraciones YAML.
    """
    pass


# Registrar todos los comandos
run.register_commands(cli)
list_jobs.register_commands(cli)


def main():
    cli()


if __name__ == '__main__':
    main()



import click
import json
import logging
from ..utils.ModuleRunner import ModuleRunner

# Configura el sistema de registro de registros
logger = logging.getLogger(__name__)


@click.command()
@click.option('--job', '-j', required=True, help='Nombre del job.')
@click.option('-p', '--param', multiple=True, help='Parámetros en formato "parametro=valor". Puedes especificar múltiples parámetros.')
def run(job, param):
    """
    Ejecuta el comando 'run' con un nombre de job y parámetros opcionales.
    """
    logger.info(f'Nombre del job: {job}')

    # Validar el formato "key=value" para todos los parámetros
    params_dict = {}
    for p in param:
        parts = p.split('=')
        if len(parts) != 2:
            logger.error(
                f'Error: El parámetro "{p}" no sigue el formato "key=value".')
            return
        key, value = parts
        params_dict[key] = value

    # Si todos los parámetros son válidos, exportar a JSON
    if params_dict:
        json_params = json.dumps(params_dict, indent=4)
        logger.debug(f'Parámetros en formato JSON:\n{json_params}')

    ModuleRunner(job).run(params_dict)


def register_commands(cli):
    cli.add_command(run)


@click.group()
def cli():
    """
    Mi aplicación de línea de comandos.
    """
    pass


# Registra el comando 'run' en la aplicación principal.
register_commands(cli)

if __name__ == '__main__':
    cli()



import jinja2

templates = {
    "default": """
tags:
  # Tagexamples
  sound1: /Users/alvaroperis/Dropbox/cinemawritter/playmix/hinds.wav
  bg: /Users/alvaroperis/Dropbox/cinemawritter/playmix/intro.wav
characters:
  {% for character in characters %}
  # {{ character.name }}
  - id: {{ character.character_id }}
    voice:
      tts:
        voiceid: 14
      filter:
        - reverb:
            room_size: 0.1
  {% endfor %}
timeline:
  {%- for scene in scenes %}
  # {{ scene.location }}
  - type: scene
    scene_id: {{ scene.scene_id }}
    background: bg
  - type: sound
    track: sound1
    duration: 8
    volume: inout
  {% endfor %}
        """
}


class TimelineTemplate:

    def __init__(self, template_string):

        if template_string not in templates:
            raise ValueError(
                f"Template key '{template_string}' not found in the templates dictionary.")

        self.template_string = template_string
        self.template = jinja2.Template(templates[template_string])

    def render(self, **kwargs):
        return self.template.render(**kwargs)
